/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// The Initial Developer of the Original Code is
// Steven Bengtson.
// Portions created by the Initial Developer are Copyright (C) 2008
// the Initial Developer. All Rights Reserved.
//
// Contributor(s):
//  Steven Bengtson <steven@songbirdnest.com>
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

// Constants for convience
if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cu) == "undefined")
  var Cu = Components.utils;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;

// Searches using engines tagged with "songbird:internal" are not
// sent to the browser
const SEARCHENGINE_TAG_INTERNAL = "songbird:internal";

// Alias identifying the Songbird search engine
const SEARCHENGINE_ALIAS_SONGBIRD = "songbird-internal-search";

const STRING_BUNDLE = "chrome://albumartmanager/locale/albumartmanager.properties";

const FETCH_BUTTON_ENABLED  = 1;
const FETCH_BUTTON_DISABLED = 2;
const FETCH_BUTTON_HIDDEN   = 3;

const SEARCH_FILTER = 0;
const ARTIST_FILTER = 1;
const ALBUM_FILTER  = 2;

const LOAD_TIMER_INTERVAL = 50;

Cu.import("resource://app/jsmodules/sbProperties.jsm");
Cu.import("resource://app/jsmodules/sbLibraryUtils.jsm");
Cu.import("resource://app/jsmodules/StringUtils.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

var AlbumArtManager = {
  className: "sbAlbumArtManager",
  
  _bundle: null,            // String bundle
  _preferences: null,       // Preferences Service
  
  _mainView: null,          // View we are using of the main library
  _cfs: null,               // CascadeFilterSet of _mainView
  
  _loadTimer: null,         // Timer for loading the albums
  _loadCount: 0,
  _loadIndex: 0,
  
  // set up a timer to do work on
  _searchTimer: null,
  _searchNodes: [],         // Array of guids to search for covers
  _searchNodeLength: 0,     // Number of nodes we searched for
  
  DEBUG: false,             // Set to true for debug output on console
  
  /**
   * Internal debugging functions
   */
  /**
   * \brief Dumps out a message if the DEBUG flag is enabled with
   *        the className pre-appended.
   * \param message String to print out.
   */
  _debug: function (message)
  {
    if (!this.DEBUG) return;
    try {
      dump(this.className + ": " + message + "\n");
      this._consoleService.logStringMessage(this.className + ": " + message);
    } catch (err) {
      // We do not want to throw exception here
    }
  },
  
  /**
   * \brief Dumps out an error message with the className + ": [ERROR]"
   *        pre-appended, and will report the error so it will appear in the
   *        error console.
   * \param message String to print out.
   */
  _logError: function (message)
  {
    try {
      dump(this.className + ": [ERROR] - " + message + "\n");
      Cu.reportError(this.className + ": [ERROR] - " + message);
    } catch (err) {
      // We do not want to thow exceptions here
    }
  },

  /**
   * AlbumArtManager functions
   */
  init: function() {
    this._mainLibrary = LibraryUtils.mainLibrary;

    this._preferences = Cc['@mozilla.org/preferences-service;1']
                        .getService(Ci.nsIPrefBranch);

    this._consoleService = Cc["@mozilla.org/consoleservice;1"]
                            .getService(Ci.nsIConsoleService);

    try {
      this.DEBUG = this._preferences.getBoolPref("songbird.albumartmanager.debug");
    } catch (err) {}
    this.DEBUG=true;
    
    var stringBundleService = Cc["@mozilla.org/intl/stringbundle;1"]
                              .getService(Ci.nsIStringBundleService);
    this._bundle = stringBundleService.createBundle(STRING_BUNDLE);

    // Listen for search events
    document.addEventListener("aamsearch",
                              function (event) {
                                AlbumArtManager.onSearchEvent(event);
                              }, 
                              true);        

    // Create a view for the Manager
    this._mainView = this._mainLibrary.createView();
    this._cfs = this._mainView.cascadeFilterSet;
    
    // Create a search
    this._cfs.appendSearch([
                            SBProperties.artistName,
                            SBProperties.albumName
                           ],
                           2);
    
    // Create a filter set for artist and album.
    this._cfs.appendFilter(SBProperties.artistName);
    this._cfs.appendFilter(SBProperties.albumName);
    
    // Listen to changes to the filters so we can rebuild the album list
    this._cfs.addListener(this);
    
    // Display the artists in a filter box
    var artistFilterBox = document.getElementById("aamArtistListFilterBox");
    artistFilterBox.bind(this._mainView, ARTIST_FILTER);
    artistFilterBox.tree.setAttribute("hidecolumnpicker", true);

    // Build the list of albums (using the filter at index 2)
    this._loadCount = this._cfs.getValueCount(ALBUM_FILTER) - 1;
    this.updateStatus("Loading albums", 0, this._loadCount);
    this._loadTimer = Cc["@mozilla.org/timer;1"].createInstance(Ci.nsITimer);
    this._loadTimer.initWithCallback(this,
                                     LOAD_TIMER_INTERVAL, 
                                     Ci.nsITimer.TYPE_REPEATING_SLACK);

  },
  
  deinit: function() {
    // Unload listeners and services
    if (this._loadTimer) {
      this._loadTimer.cancel();
      this._loadTimer = null;
    }
    this._bundle = null;
    this._preferences = null;
  },

  openPrefs: function() {
    if (window && window.SBOpenPreferences) {
      window.SBOpenPreferences("library_albumartmanager_pref_pane");
    } else {
      var windowMediator;
      var sbWindow;

      /* Get the main Songbird window. */
      windowMediator = Components.classes
                          ["@mozilla.org/appshell/window-mediator;1"].
                          getService(Components.interfaces.nsIWindowMediator);
      sbWindow = windowMediator.getMostRecentWindow("Songbird:Main");

      /* Open the preferences pane. */
      sbWindow.SBOpenPreferences("library_albumartmanager_pref_pane");
    }
  },

  closeWindow: function() {
    window.close();
  },

  /**
   * UI Updating functions
   */
  updateStatus: function(aMessage, aIndex, aTotalCount) {
    var mStatusLabel = document.getElementById("AlbumArtManager_StatusMessage");
    var mStatusProgress = document.getElementById("AlbumArtManager_StatusProgress");
    var mStatusProgressLabel = document.getElementById("AlbumArtManager_StatusProgressLabel");
    
    mStatusLabel.setAttribute("label", aMessage);
    
    if (aIndex == 0 && aTotalCount == 0) {
      mStatusProgress.setAttribute("hidden", "true");
      mStatusProgressLabel.setAttribute("hidden", "true");
    } else {
      var percentage = Math.round(((aIndex / aTotalCount) * 100));
      mStatusProgress.setAttribute("value", percentage);
      mStatusProgressLabel.setAttribute("label",
                                        SBFormattedString("albumartmanager.status.searchingpercent",
                                                          [aIndex, aTotalCount, percentage],
                                                          this._bundle));
    }
  },

  _updateFetchButton: function(aState) {
    var fetchButton = document.getElementById("toolbar-missing-covers");
    var cancelButton = document.getElementById("toolbar-missing-covers-cancel");
    
    switch (aState) {
      case FETCH_BUTTON_ENABLED:
        this._debug("Updating fetch button: [ENABLED] " + aState);
        cancelButton.setAttribute("hidden", true);
        fetchButton.removeAttribute("disabled");
        fetchButton.removeAttribute("hidden");
      break;
      case FETCH_BUTTON_DISABLED:
        this._debug("Updating fetch button: [ENABLED] " + aState);
        cancelButton.setAttribute("hidden", true);
        fetchButton.setAttribute("disabled", true);
        fetchButton.removeAttribute("hidden");
      break;
      case FETCH_BUTTON_HIDDEN:
        this._debug("Updating fetch button: [ENABLED] " + aState);
        cancelButton.removeAttribute("hidden");
        fetchButton.setAttribute("disabled", true);
        fetchButton.setAttribute("hidden", true);
      break;
      default:
        this._debug("Unkown state for fetch button " + aState);
        this.updateFetchButton(FETCH_BUTTON_ENABLED);
      break;
    }
  },
  
  _toggleFiltering: function(isEnabled) {
    var artistFilterBox = document.getElementById("aamArtistListFilterBox");
    var searchBar = document.getElementById("searchbar");

    if (isEnabled) {
      artistFilterBox.removeAttribute("disabled");
      searchBar.removeAttribute("disabled");
    } else {
      artistFilterBox.setAttribute("disabled", true);
      searchBar.setAttribute("disabled", true);      
    }
  },
  
  _getAlbumDetails: function(aAlbumTitle, aAlbumElement) {
    var self = this;
    var mediaListListener = {
      onEnumerationBegin: function() {
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumeratedItem: function(list, item) {
        aAlbumElement.addItemToAlbum(item);
        return Ci.sbIMediaListEnumerationListener.CONTINUE;
      },
      onEnumerationEnd: function() {
      }
    };

    var propertyArray =
      Cc["@songbirdnest.com/Songbird/Properties/MutablePropertyArray;1"]
      .createInstance(Ci.sbIMutablePropertyArray);
    propertyArray.appendProperty(SBProperties.albumName, aAlbumTitle);

    var mediaList = this._mainView.mediaList;
    mediaList.enumerateItemsByProperties(propertyArray,
                                         mediaListListener,
                                         Ci.sbIMediaList
                                           .ENUMERATIONTYPE_SNAPSHOT);
  },
  
  _addNextAlbum: function() {
    // Do this the hard way first    
    if (this._loadIndex < this._loadCount) {
      var cfs = this._mainView.cascadeFilterSet;
      var itemValue = cfs.getValueAt(ALBUM_FILTER, this._loadIndex);
      this.updateStatus(itemValue, this._loadIndex, this._loadCount);
      this._loadIndex++;

      // Ok we have an album name, how do we get more information?
      var albumListBox = document.getElementById("aamAlbumListBox");
      var albumElement = document.createElement("albumplace");
      albumElement.libraryRef = this._mainLibrary;
      albumListBox.appendChild(albumElement);
      
      this._getAlbumDetails(itemValue, albumElement);
    } else {
      this.updateStatus("Finished loading " + this._loadCount + " albums", 0, 0);
      this._loadTimer.cancel();
      this._loadTimer = null;
      //this._updateFetchButton(FETCH_BUTTON_ENABLED);
      this._toggleFiltering(true);
    }
  },

  _filterAlbums: function() {
    this._debug("_filterAlbums Called");
    // Since the filterlist has changed we need to hide/show the album entries
    var albumListBox = document.getElementById("aamAlbumListBox");
    if (albumListBox.childNodes.length <= 0) {
      // Do not do anything if there are no albums in the list.
      this._debug("No albums in the list!");
      return;
    }

    var albumElement = albumListBox.firstChild;    
    var listLength = this._cfs.getValueCount(ALBUM_FILTER);
    this._debug("There are now " + listLength + " albums to show");
    for (var i = 0; i < listLength && albumElement; i++) {
      var albumName = this._cfs.getValueAt(ALBUM_FILTER, i);

      // Hide any elements that are before this one and after the last one
      while (albumElement && albumElement.albumName != albumName) {
        albumElement.setAttribute("collapsed", true);
        albumElement = albumElement.nextSibling;
      }
      
      if (albumElement && albumElement.albumName == albumName) {
        albumElement.removeAttribute("collapsed");
        albumElement = albumElement.nextSibling;
      }
    }

    // Hide the rest of the elements
    while (albumElement) {
      albumElement.setAttribute("collapsed", true);
      albumElement = albumElement.nextSibling;
    }
  },


  /*********************************
   * Search Handler
   ********************************/
  /**
    * Called when a "aamsearch" event is received. 
    * Search events are expected to come from elements
    * with .value and .currentEngine properties.
    */  
  onSearchEvent: function ( evt )
  {
    // Find the search widget responsible for this event
    var widget = this._getSearchEventTarget(evt);
    if (widget == null) {
      this._logError("Could not process search event. " +
            "Target did not have a currentEngine property.");
      return;
    }

    // If this engine has not been tagged as internal
    // then dispatch the search normally
    if ( widget.currentEngine.tags.indexOf(SEARCHENGINE_TAG_INTERNAL) != -1 ) 
    { 
      // Is this our internal search?  If not, do nothing.  
      // Other people can add their own listeners.
      if ( widget.currentEngine.alias == SEARCHENGINE_ALIAS_SONGBIRD )
      {
        this._doLibrarySearch(widget.value);
      } 
    }
  },

  /**
   * Get the real target of this event
   * (hack through binding and browser layers)
   */
  _getSearchEventTarget: function (evt)  {
    var target;
    
    // If normal search event
    if (evt.target && evt.target.currentEngine) {
      target = evt.target;
    // If search target is within a binding
    } else if (evt.originalTarget && evt.originalTarget.currentEngine) {
      target = evt.originalTarget;
    // If search target is within a browser document
    } else if (evt.target && evt.target.wrappedJSObject && 
              evt.target.wrappedJSObject.currentEngine) 
    {
      target = evt.target.wrappedJSObject;        
    // If search target is within a browser document AND a binding
    } else if (evt.originalTarget && evt.originalTarget.wrappedJSObject && 
              evt.originalTarget.wrappedJSObject.currentEngine) 
    {
      target = evt.originalTarget.wrappedJSObject;        
    // Else I'm out of ideas...
    } else {
      this._logError("Error! Search event received from a target with no currentEngine!");             
      try {
        this._logError("\ttarget " + evt.target.tagName);
      } catch (e) {};
      try {
        this._logError("\toriginalTarget " + evt.originalTarget.tagName);
      } catch (e) {};
    }
    return target;
  },

  /**
   * Attempt to set a search filter on the media page in the current tab.
   * Returns true on success.
   */
  _doLibrarySearch: function (query) {
    /* we need an sbIMediaListView with a cascadeFilterSet whose first
     * filter is a search */
    if (!this._cfs ||
        this._cfs.length < 1 ||
        !this._cfs.isSearch(SEARCH_FILTER)) {
      return;
    }

    // The search is always the first filter in the cascade set
    if (query == "" || query == null) {
      this._cfs.set(SEARCH_FILTER, [], 0);
    } else {
      var stringTransform = 
        Components.classes["@songbirdnest.com/Songbird/Intl/StringTransform;1"]
                  .createInstance(Components.interfaces.sbIStringTransform);
                  
      var newQuery = stringTransform.normalizeString("",
                            Ci.sbIStringTransform.TRANSFORM_IGNORE_NONSPACE,
                            query);
      query = newQuery;      

      var valArray = query.split(" ");
      this._cfs.set(SEARCH_FILTER, valArray, valArray.length);
    }
  },

  /*********************************
   * sbICascadeFilterSetListener
   ********************************/
  onChange: function() {
    this._debug("onChange");
  },

  onValuesChanged: function(aIndex) {
    if (aIndex == ALBUM_FILTER &&
        !this._loadTimer) {
      this._filterAlbums();
    }
  },

  /*********************************
   * nsITimerCallback
   ********************************/  
  notify: function(aTimer) {
    if (aTimer == this._loadTimer) {
      this._addNextAlbum();
    }
  },

  /*********************************
   * nsISupports
   ********************************/
  QueryInterface: XPCOMUtils.generateQI([Ci.nsISupports,
                                         Ci.nsITimerCallback,
                                         Ci.sbICascadeFilterSetListener])

};
