/*
//
// BEGIN SONGBIRD GPL
//
// This file is part of the Songbird web player.
//
// Copyright(c) 2005-2008 POTI, Inc.
// http://songbirdnest.com
//
// The Initial Developer of the Original Code is
// Steven Bengtson.
// Portions created by the Initial Developer are Copyright (C) 2008
// the Initial Developer. All Rights Reserved.
//
// Contributor(s):
//
// This file may be licensed under the terms of of the
// GNU General Public License Version 2 (the "GPL").
//
// Software distributed under the License is distributed
// on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, either
// express or implied. See the GPL for the specific language
// governing rights and limitations.
//
// You should have received a copy of the GPL along with this
// program. If not, go to http://www.gnu.org/licenses/gpl.html
// or write to the Free Software Foundation, Inc.,
// 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
//
// END SONGBIRD GPL
//
*/

// Constants for convience
if (typeof(Cc) == "undefined")
  var Cc = Components.classes;
if (typeof(Ci) == "undefined")
  var Ci = Components.interfaces;
if (typeof(Cu) == "undefined")
  var Cu = Components.utils;
if (typeof(Cr) == "undefined")
  var Cr = Components.results;

const STRING_BUNDLE = "chrome://albumartmanager/locale/albumartmanager.properties";

Cu.import("resource://app/jsmodules/sbProperties.jsm");
Cu.import("resource://app/jsmodules/sbLibraryUtils.jsm");
Cu.import("resource://app/jsmodules/StringUtils.jsm");
Cu.import("resource://gre/modules/XPCOMUtils.jsm");

/**
 * Since we can't use the FUEL components until after all other components have
 * been loaded we define a lazy getter here for when we need it.
 */
__defineGetter__("Application", function() {
  delete this.Application;
  return this.Application = Cc["@mozilla.org/fuel/application;1"]
                              .getService(Ci.fuelIApplication);
});

/**
 * \brief albumArHelper is a component that has helper functions for album art
 *        operations.
 */
function albumArthelper () {
  this.DEBUG = Application.prefs.getValue("songbird.albumartmnager.debug", false);
	if (this.DEBUG) {
    this._consoleService = Cc["@mozilla.org/consoleservice;1"]
                            .getService(Ci.nsIConsoleService);
  }
};

albumArthelper.prototype = {
  className: "albumArtHelper",
  constructor: albumArthelper,       // Constructor to this object
  classDescription: "Songbird Album Art Helper",
  classID: Components.ID("{fd6f3329-9023-4ea5-9a98-34331338fe44}"),
  contractID: "@songbirdnest.com/albumart/helper;1",

  // Variables
  DEBUG: false,                       // Debug flag for _debug function
  _fetchers: null,                      // List of available fetchers
  
  /**
   * Internal debugging functions
   */
  /**
   * \brief Dumps out a message if the DEBUG flag is enabled with
   *        the className pre-appended.
   * \param message String to print out.
   */
  _debug: function (message) {
    if (!this.DEBUG) return;
    try {
      dump(this.className + ": " + message + "\n");
      this._consoleService.logStringMessage(this.className + ": " + message);
    } catch (err) {
      // We do not want to throw an exception here
    }
  },
  
  /**
   * \brief Dumps out an error message with the className + ": [ERROR]"
   *        pre-appended, and will report the error so it will appear in the
   *        error console.
   * \param message String to print out.
   */
  _logError: function (message) {
    try {
      dump(this.className + ": [ERROR] - " + message + "\n");
      Cu.reportError(this.className + ": [ERROR] - " + message);
    } catch (err) {
      // We do not want to thow an exception here
    }
  },

	/**
	 * \brief Adds a fetcher to the list of fetchers in proper priority order
	 * \param aContractID Contract ID of the fetcher to add
	 */
  _addFetcher: function(aContractID) {
    try {
      this._debug("Trying to add Fetcher " + aContractID);
      var fetcherService = Cc[aContractID].getService(Ci.sbICoverFetcher);
      // Get the priority of this one and then put it in the proper spot in the
      // array
      var curPriority = 99;
      try {
        curPriority = this._prefService.getIntPref("songbird.albumartmanager.fetcher." +
                                                   fetcherService.shortName +
                                                   ".priority");
      } catch (err) {}
      
      var fIndex = 0;
      for (fIndex = 0; fIndex < this._fetchers.length; fIndex++) {
        var fetcherCid = this._fetchers
                     .queryElementAt(fIndex, Ci.nsISupportsString)
                     .toString();
        var nFetcherService = Cc[fetcherCid].getService(Ci.sbICoverFetcher);

        var fPriority = 99;
        try {
          // TODO: Change this to use nFetcherService.priority instead
          fPriority = this._prefService.getIntPref("songbird.albumartmanager.fetcher." +
                                                   nFetcherService.shortName +
                                                   ".priority");
        } catch (err) {}

        if (fPriority > curPriority) {
          break; // jump out so we can insert
        }
      }

      var cidString =  Cc["@mozilla.org/supports-string;1"]
                         .createInstance(Ci.nsISupportsString);
      cidString.data = aContractID;
      if (fIndex < this._fetchers.length) {
        this._debug("Inserting fetcher [" + aContractID + "] as position " + fIndex);
        this._fetchers.insertElementAt(cidString, fIndex, false);
      } else {
        this._debug("Appending fetcher [" + aContractID + "]");
        // Didn't find a place for it so just append
        this._fetchers.appendElement(cidString, false);
      }
    } catch (err) {
      // Bad service just ignore
      this._logError("Unable to add Fetcher: " + aContractID);
    }
  },
  
  /**
   * \brief creats a list of fetchers and orders them based on priority
   */
  _loadFetcherList: function() {
    this._debug("Creating list of fetcher components");
    var catManager = Cc['@mozilla.org/categorymanager;1']
                       .getService(Ci.nsICategoryManager);
    var catEnumerator = catManager.enumerateCategory('sbCoverFetcher');
    this._fetchers = Cc["@mozilla.org/array;1"]
                       .createInstance(Ci.nsIMutableArray);
    while (catEnumerator.hasMoreElements()) {
      try {
        var entry = catEnumerator
                     .getNext()
                     .QueryInterface(Ci.nsISupportsCString);
        var contractID = catManager.getCategoryEntry('sbCoverFetcher', entry);
        // Now we need to determine the priority of this fetcher
        this._addFetcher(contractID);
      } catch (err) {
        this._logError("Failed to get fetcher: " + err);
      }
    }
  },

  /*********************************
   * albumArtHelper
   ********************************/
  /**
   * \brief Returns an array of contract ids for the available fetchers.
   * \returns nsIArray array of nsISupportsString
   */
  getFetcherList: function() {
		this._debug("getFetcherList called");
    if (!this._fetchers) {
			this._debug("Loading Fetchers");
			this._fetchers = [];
      this._loadFetcherList();
    }
    return this._fetchers;
  },
  
  /*********************************
   * nsISupports
   ********************************/
  QueryInterface: XPCOMUtils.generateQI([Ci.albumArtHelper])
}

function NSGetModule(compMgr, fileSpec) {
  return XPCOMUtils.generateModule([albumArthelper]);
}